using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace InchWest.HotkeyControl
{
  //It is essential to implement the HotkeyControl plugin interface
  public class ScreenShot : IHCtrl
  {
    #region Api Members
    [DllImport("user32.dll")]
    private static extern int GetForegroundWindow();
    [DllImport("user32.dll")]
    private static extern int GetDesktopWindow();
    [DllImport("user32.dll")]
    private static extern int GetDC(int hwnd);
    [DllImport("gdi32.dll")]
    private static extern int BitBlt(int hDestDC, int x, int y, int nWidth, int nHeight, int hSrcDC, int xSrc, int ySrc, int dwRop);
    [DllImport("user32.dll")]
    private static extern int GetWindowRect(int hwnd, ref RECT lpRect);

    [StructLayout(LayoutKind.Sequential)]
    private struct RECT
    {
      internal int Left;
      internal int Top;
      internal int Right;
      internal int Bottom;
    }
    #endregion

    #region IHCtrl Members

    private string[] pluginParams;

    public void Load(string[] pluginParam)
    {
      //Keep a copy of common plugin parameters
      pluginParams = pluginParam;
    }

    public void ConfigPlugin(ref string[] hotkeyParam, ref string[] pluginParam)
    {
      ScreenConfigFrm configForm = new ScreenConfigFrm();
      //Check for null parameters for initial configuration
      if (hotkeyParam == null || hotkeyParam.Length==0)
        hotkeyParam = new string[1] { "True" };
      if (pluginParam == null || pluginParam.Length == 0)
        pluginParam = new string[2] { "", "Jpeg (High)" };
      //Set the UI elements using data from the parameters
      configForm.CapR1.Checked = Convert.ToBoolean(hotkeyParam[0]);
      configForm.FolderTxt.Text = pluginParam[0];
      configForm.ImgTypeC.SelectedItem = pluginParam[1];
      //Show a modal form and make the modification to parameters if OK button was pressed
      if (configForm.ShowDialog() == DialogResult.OK)
      {
        hotkeyParam = new string[1] { configForm.CapR1.Checked.ToString() };
        pluginParam = new string[2] { configForm.FolderTxt.Text, configForm.ImgTypeC.SelectedItem.ToString() };
      }
    }

    public string Display(string[] hotkeyParam)
    {
      //Don't show any on screen display because it will also get captured
      return "";
    }

    public void Run(string[] hotkeyParam)
    {
      String path;
      if (hotkeyParam == null || hotkeyParam.Length == 0)
      {
        //Check for null to make sure the plugin was configured
        MessageBox.Show("You have not configured Screenshot plugin yet. Please configure it from the settings window so that you can use it.",
          "Screenshot plugin", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
      }
      else
      {
        //Capture either active window or the desktop
        RECT rect = new RECT();
        if(Convert.ToBoolean(hotkeyParam[0])!=true)
          GetWindowRect(GetDesktopWindow(), ref rect);
        else
          GetWindowRect(GetForegroundWindow(), ref rect);
        if (rect.Bottom + rect.Top + rect.Right + rect.Left == 0)
        {
          GetWindowRect(GetDesktopWindow(), ref rect);
        }
        Bitmap b = new Bitmap(rect.Right-rect.Left, rect.Bottom-rect.Top);
        Graphics g = Graphics.FromImage(b);
        System.IntPtr h2 = g.GetHdc();
        BitBlt(h2.ToInt32(), 0, 0, b.Width,b.Height, GetDC(GetDesktopWindow()), rect.Left,rect.Top, 0x00CC0020);
        g.ReleaseHdc(h2);

        if (pluginParams[0] == "")
        {
          //Show a file dialog if the user did not choose to autosave the screenshot
          SaveFileDialog filedialog = new SaveFileDialog();
          filedialog.Title = "Save the screenshot to";
          filedialog.OverwritePrompt = true;
          if (pluginParams[1].Equals("Bitmap"))
            filedialog.Filter = "Bitmap file (*.bmp)|*.bmp";
          else
            filedialog.Filter = "Jpeg file (*.jpg)|*.jpg";
          //Show dialog with desktop as parent so that it is visible in the taskbar
          if (filedialog.ShowDialog(new Win32Window(GetDesktopWindow())) != DialogResult.OK)
            return;
          path = filedialog.FileName;
        }
        else
        {
          //Create a filename based on the current date, time
          if (pluginParams[1].Equals("Bitmap"))
            path = pluginParams[0] + "\\Cap" + DateTime.Now.ToString("yyyy-MM-dd_HH-mm-ss") + ".bmp";
          else
            path = pluginParams[0] + "\\Cap" + DateTime.Now.ToString("yyyy-MM-dd_HH-mm-ss") + ".jpg";
        }

        //Save the file in one of the two formats
        if (pluginParams[1].Equals("Bitmap"))
        {
          b.Save(path,ImageFormat.Bmp);
        }
        else
        {
          EncoderParameters enc = new EncoderParameters(1);
          if (pluginParams[1].Equals("Jpeg (High)"))
            enc.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 100L);
          if (pluginParams[1].Equals("Jpeg (Medium)"))
            enc.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 75L);
          if (pluginParams[1].Equals("Jpeg (Low)"))
            enc.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 50L);
          ImageCodecInfo[] ic = ImageCodecInfo.GetImageEncoders();
          foreach (System.Drawing.Imaging.ImageCodecInfo ici in ic)
          {
            if (ici.MimeType == "image/jpeg")
            {
              b.Save(path, ici, enc);
              break;
            }
          }
        }
      }
    }

    public void UnLoad()
    {
      //Nothing to unload, can leave blank
    }

    #endregion
  }

  //Class implementation to display savedialog with desktop as owner, only needed for
  //this sample, you don't need this for implementing your plugin.
  internal class Win32Window : IWin32Window
  {
    public Win32Window(object handle)
    {
      _hwnd = (IntPtr)Convert.ToInt32(handle);
    }

    public IntPtr Handle
    {
      get { return _hwnd; }
    }

    private IntPtr _hwnd;
  }
}
